use std::io::{BufRead, Cursor, Error, Read, Write};
use std::net::{TcpListener, TcpStream};

use byteorder::{BigEndian, ReadBytesExt, WriteBytesExt};
use rust_ofp::learning_switch::LearningSwitch;
use rust_ofp::ofp_controller::OfpController;

extern crate byteorder;
extern crate rust_ofp;

struct packetIn {
    buf_id: Option<i32>,
    payload: Vec<u8>,
    total_len: u16,
    port: u16,
    reason: u8,
}

fn packetin_payload(buf:&mut Vec<u8>) -> Result<packetIn, Error> {
    let mut packet = Cursor::new(buf.to_vec());
    let buf_id = match packet.read_i32::<BigEndian>()? {
        -1 => None,
        n => Some(n),
    };
    let total_len = packet.read_u16::<BigEndian>()?;
    let port = packet.read_u16::<BigEndian>()?;
    let reason = packet.read_u8()?;
    packet.consume(1);
    let payload = packet.fill_buf()?.to_vec();
    Ok(packetIn {
        buf_id: buf_id,
        payload: payload,
        port: port,
        reason: reason,
        total_len: total_len,
    })
}

fn send_hello(stream: &mut TcpStream) {
    let mut buf: Vec<u8> = Vec::new();
    buf.write_u8(1).unwrap();
    buf.write_u8(0).unwrap();
    buf.write_u16::<BigEndian>(8).unwrap();
    buf.write_u32::<BigEndian>(0).unwrap();
    println!("send: {:?}", buf);
    stream.write_all(&buf).unwrap();
}

fn feture_req(stream: &mut TcpStream, xid: u32) -> Result<(), std::io::Error> {
    let mut buf: Vec<u8> = Vec::new();
    buf.write_u8(1)?;
    buf.write_u8(5)?;
    buf.write_u16::<BigEndian>(8)?;
    buf.write_u32::<BigEndian>(xid)?;
    println!("send: {:?}", buf);
    stream.write_all(&buf)?;
    Ok(())
}

fn main() -> Result<(), std::io::Error> {
    let listener = TcpListener::bind(("127.0.0.1", 6633)).unwrap();
    let mut buf = vec![0u8; 8];
    for stream in listener.incoming() {
        println!("{:?}", stream);
        match stream {
            Ok(mut stream) => {
                // std::thread::spawn(move || {
                println!("=================== connection =======================");

                // LearningSwitch::handle_client_connected(&mut stream);
                // continue;

                // after tcp is connected, it will send hello message
                send_hello(&mut stream);

                // loop for receive data
                loop {
                    // first receive with Openflow header 64 bit to buf
                    let res = stream.read(&mut buf);
                    match res {
                        Ok(v) if v > 0 => {
                            let mut packet = Cursor::new(buf.to_vec());
                            println!("buf: {:?}", packet);
                            // split data from header
                            let _version = packet.read_u8().unwrap();
                            let message = packet.read_u8().unwrap();
                            // length is only payload size to receive
                            let length = packet.read_u16::<BigEndian>().unwrap() - 8;
                            let xid = packet.read_u32::<BigEndian>().unwrap();

                            // message_body is var to receive payload if it has
                            // and assign size by length
                            let mut message_body = vec![0u8; length as usize];
                            stream.read(&mut message_body)?;

                            match message {
                                // 0 is Hello message
                                0 => {
                                    // after get Hello, send fetureReq
                                    feture_req(&mut stream, xid)?;
                                }
                                10 => {
                                    let payload = packetin_payload(&mut buf)?;
                                }
                                _ => {
                                    println!("others message");
                                }
                            }
                        }
                        Ok(_) | Err(_) => break,
                    }
                }
                println!("======================================================");

                // });
            }
            Err(_) => {
                // connection failed
                panic!("Connection failed")
            }
        }
    }
    Ok(())
}
